' Remove registry settings and shortcuts for LAN TeX Live installation,
' including path components and file associations.
' It has been copied to the user's profile upon installation.

Option Explicit

On Error Resume Next

Dim cfgdir, sTL, Menu_Name, rel_year, TL_Menu, Appdata_Dir
Dim exts, pid_exts, ftypes

Const SpecFoldKey = _
  "HKCU\software\microsoft\windows\currentversion\explorer\user shell folders"

' load necessary functionality

Dim oWsh, oFS, oReg, regerror

Const hkcu_id = &h80000001 ' registry hive HKCU

Set oWsh = Wscript.CreateObject("WScript.Shell")
Set oFS = CreateObject("Scripting.FileSystemObject")

' WMI registry access, needed for enumerating subkeys (!)

regerror = False
Set oReg = GetObject("winmgmts:{impersonationLevel=impersonate}//" _
    & "./root/default:StdRegProv")
If Err.number <> 0 Then
  Err.clear
  regerror = True
End If

''''''''''''''''''''''''''''''''''''''''
Sub DisplayMessage( m )

  ' while debugging from command prompt:
  ' wscript.echo m
  ' production version:
  MsgBox m,, Menu_Name & " uninstaller"
End Sub

'''''''''''''''''''''''''''''''''''''''''
' uppercase and fix pathname for reliable comparison

Function std_path( s )
  On Error Resume next
  std_path = UCase(replace( s, "/", "\" ))
  std_path = replace( std_path, ":\\", ":\" )
  If InStr( std_path, """") = 1 Then
    std_path = Mid( std_path, 2, Len(std_path)-2)
  End If
End Function

''''''''''''''''''''''''''
' delete a file. Return true if success or file did not exist.

function del_file( f )

  err.clear
  if not oFS.FileExists( f ) then
    del_file = True
  else
    err.clear
    oFS.deletefile f, True ' true -> ignore read-only
    if oFS.FileExists( f ) then
      err.clear
      del_file = False
    else
      del_file = True
    end if
  end if

end function

' same for folder

function del_folder( f )

  err.clear
  if not oFS.FolderExists( f ) then
    del_folder = True
  else
    err.clear
    ' deletefolder works recursively
    oFS.deletefolder f, True ' true -> ignore read-only
    if oFS.FolderExists( f ) then
      err.clear
      ' DisplayMessage f & " still exists"
      del_folder = False
    else
      del_folder = True
    end if
  end if

end Function

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function DeleteRegBranch( sHive, sBranch )

  Dim resnum, resbool, sKeyName, sNames

  ' Delete an existing key.
  ' clear any error conditions before returning

  On Error Resume Next

  resnum = oReg.DeleteKey(sHive, sBranch)
  If ( Not Err ) And (resnum = 0 ) Then
    'DisplayMessage "deleting " & sBranch
    resbool = true
  else
    ' If the deletion failed, start deleting subkeys.
    ' No need to keep track of success;
    ' we delete what we can and check in the end
    ' whether we CAN delete the key.
    Err.clear

    ' Subkey Enumerator
    resnum = oReg.EnumKey (sHive, sBranch, sNames)
    If resnum = 0 Then
      For Each sKeyName In sNames
        'DisplayMessage "recursively deleting " & sBranch & "\" & sKeyName
        DeleteRegBranch sHive, sBranch & "\" & sKeyName
      Next
    End If
    ' no need to delete values separately

    ' Now we can try again to delete the registry key.
    ' This will fail if there was a previous failure.
    resnum = oReg.DeleteKey( sHive, sBranch )
    If Err.number <> 0 Then
      resnum = Err.number
      Err.clear
      resbool = false
    elseif resnum <> 0 Then
      resbool = False
    Else
      resbool = True
    End If
    wscript.sleep 50 ' may prevent some failures
  End If
  DeleteRegBranch = resbool

End Function

' Delete a registry value

Function reg_del_Val( s )
  On Error Resume Next
  Dim i, v
  i = InStrRev( s, "\" )
  oReg.DeleteValue hkcu_id, Mid(s,1,i), Mid(s,i+1)
  v = oWsh.RegRead( "HKCU\" & s )
  If (Err.number <> 0) Or (v = "") Then
    Err.clear
    reg_del_Val = True
  Else
    reg_del_Val = False
  End If
End Function

'''''''''''''''''''''''''''''''''''''''''''''''''
' parameters

' read parameter file

Sub read_Parameters ()

  on error resume next

  Dim oTS, l, i, p, v

  cfgdir = (oFS.GetFile(Wscript.ScriptFullName)).ParentFolder

  Set oTS = oFS.opentextfile (cfgdir & "\config.ini")
  If Err.number <> 0 Then
    Err.clear
    DisplayMessage "Cannot read data for uninstall; aborting ..."
    wscript.quit
  End If
  ' DisplayMessage "Reading " & cfgdir & "\config.ini"
  sTL = ""
  Menu_Name = "" 'name of tl menu
  Do until oTS.atendofstream
    l = oTS.readline
    i = InStr(l, "=") ' counting from 1
    p = Mid(l,1,i-1)
    v = Mid(l,i+1)
    If p = "root" Then
      sTL = std_path(v)
      ' DisplayMessage "sTL " & sTL
    elseif p = "menu" Then
      Menu_Name = v
      ' DisplayMessage "Menu_Name " & Menu_Name
    elseif p = "year" Then
      rel_year = v
      ' DisplayMessage "rel_year " & rel_Year
    Elseif p = "exts" Then ' TL possibly created an ftype for extension
      exts = v
      ' DisplayMessage "Extensions are " & exts
    elseif p = "pid_exts" Then ' TL created entry in OpenWithProgids
        pid_exts = v
      ' DisplayMessage "PID extensions are " & pid_exts
    elseif p = "ftypes" Then
        ftypes = v
      ' DisplayMessage "Filetypes are " & ftypes
    End If
  Loop
  oTS.Close

  If (Err.number <> 0) Or (sTL = "") Or (Menu_Name = "") Or (rel_year = "") _
      Or (exts = "") Or (pid_exts = "") Or (ftypes = "") then
    Err.clear
    DisplayMessage "Incomplete parameter file; aborting..."
    wscript.quit
  End If

End Sub

Function get_fprog( ftype )
  On Error Resume Next
  get_fprog = ""
  If ftype<>"" then
    get_fprog =  oWsh.RegRead( "HKCU\software\classes\" & ftype & _
      "\shell\open\command\")
    If Err.number <> 0 Then
      Err.clear
      get_fprog = ""
    End If
  End If
  ' mess = mess & "found prog " & get_fprog & " for " & ftype & vbcrlf
End Function

Function under_tl( s )
  Dim t
  On Error Resume Next
  t = std_path( s )
  ' wscript.echo "under_tl " & t
  If t = "" Then
    under_tl = False
  ElseIf InStr(t, sTL) = 1 Then
    under_tl = True
  Else
    Err.clear
    under_tl = False
  End If
End Function

Function remove_file_assocs ()

  On Error Resume Next
  Dim t, ftype, ext, ndel, cmd, result, deleted_keys()

  If regerror Then ' false if WMI access didn't work
    remove_file_assocs = False
  Else
    remove_file_assocs = True
    ndel = 0
    ' inspecting all subkeys of HKCU\software\classes may be too slow.
    ' instead, the installer gets filetypes and extensions from .tlpobj files
    ReDim deleted_keys(UBound(split(ftypes)))
    For Each ftype in split(ftypes)
      cmd = std_path(Get_fprog(ftype))
      ' wscript.echo ftype & ": " & cmd
      If under_tl(cmd) Then
        ' wscript.echo ftype & " to be deleted"
        ' this ftype to be deleted
        deleted_keys(ndel) = ftype
        result = DeleteRegBranch( hkcu_id, "software\classes\" & ftype )
        If Not result Then
          remove_file_assocs = False
        End If
        ndel = ndel + 1
      End If
    Next
    For Each ext in split(exts)
      t = oWsh.RegRead( "HKCU\software\Classes\" & ext & "\" )
      If Not Err Then
        For Each ftype in deleted_keys
          If t = ftype Then
            If Not reg_del_val( "software\Classes\" & ext & "\" ) Then
              remove_file_assocs = False
              Exit For
            End If
          End If
        Next
      End If
      Err.clear
      For Each ftype in deleted_keys
        If Not reg_del_Val( "software\Classes\" & ext & _
            "\OpenWithProgids\" & ftype) Then
          remove_file_assocs = false
        End If
      Next
    Next
    For Each ext in split(pid_exts)
      For Each ftype in deleted_keys
        If Not reg_del_Val( "software\Classes\" & ext & _
            "\OpenWithProgids\" & ftype) Then
          remove_file_assocs = false
        End If
      Next
    Next
  End If

End Function

''''''''''''''''''''''''''''''''''''''''''''''''
' remove user path components which reference the installation

Function remove_from_path ()

  On Error Resume Next

  Dim p, pnew, q, hk

  err.clear()
  remove_from_path = True
  hk = "HKCU\environment\path"
  p = oWsh.RegRead( hk )
  if ( not err ) and ( p <> "" ) then
    pnew = ""
    for each q in Split( p, ";", -1 )
      If Not under_tl( q ) then
        If pnew = "" Then
          pnew = q
        Else
          pnew = pnew & ";" & q
        End If
      End If
    Next ' q
    If InStr( pnew, "%" ) > 0 Then
      oWsh.RegWrite hk, pnew, "REG_EXPAND_SZ"
    ElseIf pnew = "" Then
      oWsh.RegDelete hk
    Else
      oWsh.RegWrite hk, pnew
    End If
    If Err.number <> 0 Then
      Err.clear
      remove_from_path = False
    End If
  ' Else
    ' assume path value was not there in the first place
    ' remove_from_path = True
  End If

End Function

''''''''''''''''''''''''''''''''''''''''''''''''
' Main program

Sub Main

  On Error Resume Next

  Dim complete, s, errmess
  errmess = ""
  complete = True

  read_Parameters ' aborts on failure

  TL_Menu = oWsh.ExpandEnvironmentStrings( oWsh.RegRead( _
    SpecFoldKey & "\Programs" )) & "\" & Menu_Name
  Appdata_Dir = oWsh.ExpandEnvironmentStrings( oWsh.RegRead( _
    SpecFoldKey & "\AppData" ))

  ' delete TeX Live menu
  err.clear
  errmess = errmess & "Deleting menu" & vbcrlf
  If Not del_folder(TL_Menu) Then
    complete = false
    errmess = errmess & "Menu not completely removed." & vbcrlf
  End If

  ' file associations
  Err.clear
  errmess = errmess & "Removing file assocs" & vbcrlf
  If not remove_file_assocs() Then
    complete = false
    errmess = errmess & "File associations not completely removed." & vbcrlf
  End If

  ' searchpath
  Err.clear
  errmess = errmess & "Adjusting path" & vbcrlf
  If Not remove_from_path() Then
    complete = False
    errmess = errmess & "TeX Live not removed from searchpath" & vbcrlf
  End If

  ' delete generated files
  err.clear
  errmess = errmess & "Deleting generated files" & vbcrlf
  s = oWsh.expandenvironmentstrings("%userprofile%")
  del_folder( s & "\.texlive" & rel_year )
  ' fail silently: not important enough

  ' remove directory of uninstaller, and everything in it
  Err.clear
  errmess = errmess & "Removing uninstaller" & vbcrlf
  del_folder( Appdata_Dir & "\" & Menu_Name )
  ' again, fail silently

  If Not complete then
    DisplayMessage errmess & vbcrlf & Menu_Name & " has been uninstalled." _
        & vbcrlf & vbcrlf & "Please re-login"
  Else
    DisplayMessage Menu_Name & " has been uninstalled." _
        & vbcrlf & vbcrlf & "Please re-login"
  End If

End Sub

Main